/*
  File name: CCytadelaConfig.cpp
  Copyright: (C) 2006 - 2007 Tomasz Kazmierczak

  Creation date: 14.09.2006 17:27
  Last modification date: 28.08.2007

  This file is part of Cytadela

 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *

*/

#include "CCytadelaConfig.h"
#include "CSDLClass.h"
#include "ILocalization.h"
#include <fstream>
#include <string.h>

using namespace std;

class CConfigEntry {
private:
	string mEntryName;
	string::size_type findValuePosition(string &src);
public:
	CConfigEntry() {};
	CConfigEntry(const char *entryname) { setName(entryname); };
	void setName(const char *entryname) { mEntryName = entryname; };
	bool readValue(int32_t &dest, string &src);
	bool readValue(float &dest, string &src);
	bool readValue(char *dest, string &src, size_t destbuffsize);
};

string::size_type CConfigEntry::findValuePosition(string &src)
{
	//first find the position of the entry's name
	string::size_type iPosition = src.find(mEntryName, 0);
	if(iPosition == string::npos)
		return string::npos;
	//shift the position by the entry's name length
	iPosition += mEntryName.length();

	//now find the '=' sign
	string::size_type iEqualsPos = src.find('=', iPosition);
	if(iPosition == string::npos or iEqualsPos > iPosition)
		return string::npos;

	//and finally find the position of the value
	return src.find_first_not_of("=", iPosition);
}

bool CConfigEntry::readValue(int32_t &dest, string &src)
{
	//find the position of the entry's value
	string::size_type iPosition = findValuePosition(src);

	if(iPosition == string::npos)
		return false;
	//get the C string from the C++ string
	const char *szValue = src.c_str();
	//get the value
	dest = atoi(&szValue[iPosition]);
	return true;
}

bool CConfigEntry::readValue(float &dest, string &src)
{
	//find the position of the entry's value
	string::size_type iPosition = findValuePosition(src);
	if(iPosition == string::npos)
		return false;
	//get the C string from the C++ string
	const char *szValue = src.c_str();
	//get the value
	dest = atof(&szValue[iPosition]);
	return true;
}

bool CConfigEntry::readValue(char *dest, string &src, size_t destbuffsize)
{
	//find the position of the entry's value
	string::size_type iPosition = findValuePosition(src);
	if(iPosition == string::npos)
		return false;

	//find the position of the first quotation mark
	string::size_type iFirstQuotationMark = src.find('\"', iPosition);
	if(iFirstQuotationMark == string::npos or iFirstQuotationMark > iPosition)
		return false;
	string::size_type iValueBeginning = iFirstQuotationMark + 1;
	//and the second one
	string::size_type iSecondQuotationMark = src.find('\"', iValueBeginning);
	if(iSecondQuotationMark == string::npos)
		return false;

	//the max of chars that we can copy into dest
	size_t iCount = iSecondQuotationMark - iValueBeginning;
	if(iCount >= destbuffsize)
		iCount = destbuffsize - 1;

	memset(dest, 0, destbuffsize);
	src.copy(dest, iCount, iValueBeginning);

	return true;
}

CCytadelaConfig::CCytadelaConfig(char *szSysDirPath)
{
	//path to the config file
	sprintf(mConfigFilePath, "%s/cytadela.cfg", szSysDirPath);

	//init the game speed settings array
	static float pGameSpeedSettings[] = {1.0f, 1.1f, 1.2f, 1.3f, 1.4f, 1.5f,
										1.6f, 1.7f, 1.8f, 1.9f, 2.0f};
	mGameSpeedSettings = pGameSpeedSettings;
	//number of settings
	mNumOfGameSpeedSettings = sizeof(pGameSpeedSettings) / sizeof(float);

	//init the mouse speed settings array
	static float pMouseSpeedSettings[] = {1.0f, 2.0f, 3.0f, 4.0f, 5.0f, 6.0f,
											7.0f, 8.0f, 9.0f, 10.0f, 0.1f, 0.2f,
											0.3f, 0.4f, 0.5f, 0.6f, 0.7f, 0.8f,
											0.9f};
	mMouseSpeedSettings = pMouseSpeedSettings;
	//number of settings
	mNumOfMouseSpeedSettings = sizeof(pMouseSpeedSettings) / sizeof(float);

	//localization
	sprintf(mLocalization, CYTADELA_DEFAULT_LOCALIZATION);
	//difficulty
	mDifficult = true;
	//default game speed
	mGameSpeedSetting = 0;
	mGameSpeed = pGameSpeedSettings[0];
	//default mouse speed
	mMouseSpeedSetting = 0;
	mMouseSpeed = pMouseSpeedSettings[0];

	//try to load the config file
	ifstream ConfigFile(mConfigFilePath);
	string Configuration;
	while(ConfigFile.good() and !ConfigFile.eof())
	{
		string tmp;
		ConfigFile >> tmp;
		Configuration += tmp;
	}
	ConfigFile.close();

	//get the game's speed setting from the config
	CConfigEntry Entry("game_speed");
	float fSetting;
	if(Entry.readValue(fSetting, Configuration))
	{
		int32_t i;
		//find the loaded speed setting in the settings array
		for(i = 0; i < mNumOfGameSpeedSettings; ++i)
			if(fSetting == mGameSpeedSettings[i]) break;
		//if found then set it
		if(i < mNumOfGameSpeedSettings)
		{
			mGameSpeedSetting = i;
			mGameSpeed = fSetting;
		}
	}
	//get the mouse's speed setting from the config
	Entry.setName("mouse_speed");
	if(Entry.readValue(fSetting, Configuration))
	{
		int32_t i;
		//find the loaded speed setting in the settings array
		for(i = 0; i < mNumOfMouseSpeedSettings; ++i)
			if(fSetting == mMouseSpeedSettings[i]) break;
		//if not found then set the default speed
		if(i < mNumOfMouseSpeedSettings)
		{
			mMouseSpeedSetting = i;
			mMouseSpeed = fSetting;
		}
	}

	//get the screen resolution
	Entry.setName("game_screen_width");
	Entry.readValue(mScreenSize[0], Configuration);
	Entry.setName("game_screen_height");
	Entry.readValue(mScreenSize[1], Configuration);

	CSDLClass *sdl = CSDLClass::instance();
	const ScreenSize *pSizes = sdl->getScreenSizes();
	int32_t iScreenSizesCount = sdl->getScreenSizesCount();
	//search for actual screen size in the ScreenSizes array
	int32_t i;
	for(i = 0; i < iScreenSizesCount; ++i)
	{
		if((mScreenSize[0] == pSizes[i].iResolution[0]) and
			(mScreenSize[1] == pSizes[i].iResolution[1]))
			break;
	}
	//if not found then change to the first available
	if(i == iScreenSizesCount)
		i = sdl->findFirstAvailableScreenSize(-1);
	mScreenSize[0] = pSizes[i].iResolution[0];
	mScreenSize[1] = pSizes[i].iResolution[1];

	//get the localization name
	Entry.setName("language");
	Entry.readValue(mLocalization, Configuration, sizeof(mLocalization));
}

CCytadelaConfig::~CCytadelaConfig()
{
	ofstream ConfigFile(mConfigFilePath);

	ConfigFile << "#Cytadela configuration file\n" << endl;

	//write the configuration
	ConfigFile << "game_screen_width=" << mScreenSize[0] << ';' << endl;
	ConfigFile << "game_screen_height=" << mScreenSize[1] << ';' << endl;
	ConfigFile << "game_speed=" << mGameSpeed << ';' << endl;
	ConfigFile << "mouse_speed=" << mMouseSpeed << ';' << endl;
	ConfigFile << "language=\"" << mLocalization << "\";" << endl;

	ConfigFile.close();
}

void CCytadelaConfig::setScreenSize(int32_t iWidth, int32_t iHeight)
{
	mScreenSize[0] = iWidth;
	mScreenSize[1] = iHeight;
}

void CCytadelaConfig::setLocalization(char *szLocalization)
{
	if(strlen(szLocalization) < FILENAME_MAX)
		sprintf(mLocalization, szLocalization);
}

void CCytadelaConfig::setNextGameSpeed()
{
	mGameSpeedSetting++;
	if(mGameSpeedSetting == mNumOfGameSpeedSettings)
		mGameSpeedSetting = 0;
	mGameSpeed = mGameSpeedSettings[mGameSpeedSetting];
}

void CCytadelaConfig::setPrevGameSpeed()
{
	mGameSpeedSetting--;
	if(mGameSpeedSetting < 0)
		mGameSpeedSetting = mNumOfGameSpeedSettings-1;
	mGameSpeed = mGameSpeedSettings[mGameSpeedSetting];
}

void CCytadelaConfig::setNextMouseSpeed()
{
	mMouseSpeedSetting++;
	if(mMouseSpeedSetting == mNumOfMouseSpeedSettings)
		mMouseSpeedSetting = 0;
	mMouseSpeed = mMouseSpeedSettings[mMouseSpeedSetting];
}

void CCytadelaConfig::setPrevMouseSpeed()
{
	mMouseSpeedSetting--;
	if(mMouseSpeedSetting < 0)
		mMouseSpeedSetting = mNumOfMouseSpeedSettings-1;
	mMouseSpeed = mMouseSpeedSettings[mMouseSpeedSetting];
}
